#!/bin/sh
###
# 		dont start script as root!
#		--------------------------
#
# create example database TST with 
#    (only for seperated packages 
#              - sapdb-ind, sapdb-srv, sapdb-testdb):
#   - 20 MB data devspace and 8 MB log devspace
#   - demo database user test (with password test)
###

export PATH=/opt/sapdb/indep_prog/bin:$PATH
set -x

# name of the database
SID=ITER

# start remote communication server
# this enables us to connect with dbmcli from 
# a remote host.
x_server start >/dev/null 2>&1

# create new demo database
# create a new database. Pass the path to the software
# installation using the -R option. The -s option is
# for local operation bypassing x_server
/opt/sapdb/depend/bin/dbmcli -s -R /opt/sapdb/depend db_create $SID dbm,dbm

# create directory where to put the database files
mkdir -p $HOME/$SID

# setup database parameters
# Using a Parameter Session the new session parameters
# are stored in memory.
#
cat <<EOF | dbmcli -d $SID -u dbm,dbm
param_rmfile
param_startsession
param_init OLTP
param_put LOG_MODE SINGLE
param_put CAT_CACHE_SUPPLY 300
param_put DATA_CACHE 2500
param_put MAXDATADEVSPACES 5
param_put MAXDATAPAGES 1024000
param_checkall
param_commitsession
#
# Create the action database files. One System File,
# one datafile and one logfile. The size of the system
# file is determined automatically
#
param_adddevspace 1 SYS  $HOME/$SID/DISKS01   F
param_adddevspace 1 DATA $HOME/$SID/DISKD0001 F 2560
param_adddevspace 1 LOG  $HOME/$SID/DISKL001  F 1024
quit
EOF

# Startup database
# This puts the database in the COLD state
#
dbmcli -d $SID -u dbm,dbm db_start

# Initialize database files
# The SYSDBA user dba is created
#
cat <<EOF | dbmcli -d $SID -u dbm,dbm
util_connect dbm,dbm
util_execute init config
util_activate dba,dba
quit
EOF

# load database system tables
# The system tables are loaded. The password of
# the domain user is passed needed with the -ud option.
#
dbmcli -d $SID -u dbm,dbm load_systab -u dba,dba -ud domain

# Create database demo user of class STANDARD that is able to
# connect multiple times to the database
cat <<EOF | dbmcli -d $SID -u dba,dba
sql_connect dba,dba
sql_execute CREATE USER sima PASSWORD geheim DBA NOT EXCLUSIVE
EOF

# Define logging media
#
cat <<EOF | dbmcli -d $SID -u dbm,dbm
backup_media_put data $HOME/$SID/datasave FILE DATA 0 8 YES
backup_media_put auto $HOME/$SID/autosave FILE AUTO
util_connect dbm,dbm
backup_save data
autosave_on
quit
EOF

exit 0
